@@ -9,7 +9,7 @@ OUwie.boot <- function(phy, data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA",
 	#if alpha is NA set to a really small number -- this is only relevant for BM models:
 	alpha[is.na(alpha)]<-1e-10
 
-	cat("Beginning parametric bootstrap -- performing", nboot, "replicates", "\n")
+    cat("Beginning parametric bootstrap -- performing", nboot, "replicates", "\n")
 	
 	for(i in 1:nboot){
 		tmp.phy<-phy
@@ -24,7 +24,12 @@ OUwie.boot <- function(phy, data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA",
 			}
 		}
 		#This calls the OUwie simulator and simulates datasets:
-		tmp <- OUwie.sim(tmp.phy, data, simmap.tree=simmap.tree, scaleHeight=scaleHeight, alpha=alpha, sigma.sq=sigma.sq, theta=theta, theta0=theta0)
+        if(mserr == "none"){
+            tmp <- OUwie.sim(tmp.phy, data, simmap.tree=simmap.tree, scaleHeight=scaleHeight, alpha=alpha, sigma.sq=sigma.sq, theta=theta, theta0=theta0, mserr=mserr)
+        }
+        if(mserr == "known"){
+            tmp <- OUwie.sim(tmp.phy, data[,c(1,2,4)], simmap.tree=simmap.tree, scaleHeight=scaleHeight, alpha=alpha, sigma.sq=sigma.sq, theta=theta, theta0=theta0, mserr=mserr)
+        }
 		#OUwie.sim outputs the trait file in the order of the tree, but the trait file is likely not to be this way. So I alphabetized the input trait file above, and I do the same to the simulated trait file:
 		data <- data[order(data[,1]),]
 		tmp <- tmp[order(tmp[,1]),]
@@ -37,8 +42,18 @@ OUwie.boot <- function(phy, data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA",
 		#Now run OUwie, using the measurement error if it is contained within the data, to estimate the parameters from the simulated data:
 		tmp <- OUwie(phy, data, model=model, simmap.tree=simmap.tree, scaleHeight=scaleHeight, root.station=root.station, clade=clade, mserr=mserr, diagn=diagn, quiet=quiet, warn=warn)
 		#Now bind all the relevant output together
-		res <- rbind(res, c(tmp$solution[1,], tmp$solution[2,], tmp$theta[,1]))
+		if(model == "BM1" | model == "BMS" | model == "OU1"){
+			res <- rbind(res, c(tmp$solution[1,], tmp$solution[2,], tmp$theta[1,1]))
+		}else{
+			res <- rbind(res, c(tmp$solution[1,], tmp$solution[2,], tmp$theta[,1]))
+		}
 	}
+    if(model=="BM1" | model=="OU1"){
+        root.station=TRUE
+    }
+    if(model=="BMS"){
+        root.station=FALSE
+    }
 	if(root.station==TRUE){
 		theta.mat<-matrix(t(tmp$theta), 2, length(levels(tmp$tot.states)))
 		rownames(theta.mat)<-c("estimate", "se")
@@ -48,7 +63,11 @@ OUwie.boot <- function(phy, data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA",
 		if(tmp$simmap.tree==TRUE){
 			colnames(theta.mat) <- c(colnames(tmp$phy$mapped.edge))
 		}
-		colnames(res) <- c(paste("alpha", levels(tmp$tot.states),sep="_"),paste("sigma.sq", levels(tmp$tot.states),sep="_"),paste("theta", colnames(theta.mat),sep="_"))
+		if(model=="BM1" | model=="OU1"){
+			colnames(res) <- c(paste("alpha", levels(tmp$tot.states),sep="_"), paste("sigma.sq", levels(tmp$tot.states), sep="_"), paste("theta", "root", sep="_"))
+		}else{
+			colnames(res) <- c(paste("alpha", levels(tmp$tot.states),sep="_"), paste("sigma.sq", levels(tmp$tot.states), sep="_"), paste("theta", colnames(theta.mat), sep="_"))
+		}
 	}else{
 		theta.mat<-matrix(t(tmp$theta), 2, length(levels(tmp$tot.states))+1)
 		rownames(theta.mat)<-c("estimate", "se")
@@ -58,7 +77,11 @@ OUwie.boot <- function(phy, data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA",
 		if(tmp$simmap.tree==TRUE){
 			colnames(theta.mat)<-c("root", colnames(tmp$phy$mapped.edge))
 		}
-		colnames(res) <- c(paste("alpha", levels(tmp$tot.states),sep="_"),paste("sigma.sq", levels(tmp$tot.states),sep="_"),paste("theta", colnames(theta.mat),sep="_"))
+		if(model=="BMS"){
+			colnames(res) <- c(paste("alpha", levels(tmp$tot.states), sep="_"), paste("sigma.sq", levels(tmp$tot.states), sep="_"),paste("theta", "root", sep="_"))
+		}else{
+			colnames(res) <- c(paste("alpha", levels(tmp$tot.states), sep="_"), paste("sigma.sq", levels(tmp$tot.states), sep="_"),paste("theta", colnames(theta.mat), sep="_"))
+		}
 	}
 	class(res) <- "OUwie.boot"
 	return(res)
